@@ -235,6 +235,11 @@ fn main() {
             src_dir: "vendored_parsers/tree-sitter-make-src",
             extra_files: vec![],
         },
+        TreeSitterParser {
+            name: "tree-sitter-newick",
+            src_dir: "vendored_parsers/tree-sitter-newick-src",
+            extra_files: vec![],
+        },
         TreeSitterParser {
             name: "tree-sitter-nix",
             src_dir: "vendored_parsers/tree-sitter-nix-src",
@@ -280,6 +285,11 @@ fn main() {
             src_dir: "vendored_parsers/tree-sitter-r-src",
             extra_files: vec!["scanner.cc"],
         },
+        TreeSitterParser {
+            name: "tree-sitter-racket",
+            src_dir: "vendored_parsers/tree-sitter-racket-src",
+            extra_files: vec!["scanner.cc"],
+        },
         TreeSitterParser {
             name: "tree-sitter-ruby",
             src_dir: "vendored_parsers/tree-sitter-ruby-src",
@@ -333,7 +343,6 @@ fn main() {
     ];
 
     // Only rerun if relevant files in the vendored_parsers/ directory change.
-    println!("cargo:rerun-if-changed=vendored_parsers/highlights");
     for parser in &parsers {
         println!("cargo:rerun-if-changed={}", parser.src_dir);
     }
