@@ -67,7 +67,7 @@ use rayon::prelude::*;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::Arc;
 use std::{env, path::Path};
-use summary::{DiffResult, FileContent};
+use summary::{DiffResult, FileContent, FileFormat};
 use syntax::init_next_prev;
 use typed_arena::Arena;
 
@@ -259,6 +259,21 @@ fn main() {
     };
 }
 
+fn format_num_bytes(num_bytes: usize) -> String {
+    if num_bytes >= 1024 * 1024 * 1024 {
+        let g = num_bytes as f64 / (1024.0 * 1024.0 * 1024.0);
+        return format!("{}GiB", g.round());
+    } else if num_bytes >= 1024 * 1024 {
+        let m = num_bytes as f64 / (1024.0 * 1024.0);
+        return format!("{}MiB", m.round());
+    } else if num_bytes >= 1024 {
+        let k = num_bytes as f64 / 1024.0;
+        return format!("{}KiB", k.round());
+    }
+
+    format!("{}B", num_bytes)
+}
+
 /// Print a diff between two files.
 fn diff_file(
     lhs_display_path: &str,
@@ -284,6 +299,29 @@ fn diff_file(
     )
 }
 
+fn check_only_text(
+    file_format: &FileFormat,
+    lhs_display_path: &str,
+    rhs_display_path: &str,
+    lhs_src: &str,
+    rhs_src: &str,
+) -> DiffResult {
+    let has_changes = lhs_src != rhs_src;
+
+    return DiffResult {
+        lhs_display_path: lhs_display_path.into(),
+        rhs_display_path: rhs_display_path.into(),
+        file_format: file_format.clone(),
+        lhs_src: FileContent::Text(lhs_src.into()),
+        rhs_src: FileContent::Text(rhs_src.into()),
+        lhs_positions: vec![],
+        rhs_positions: vec![],
+        hunks: vec![],
+        has_byte_changes: has_changes,
+        has_syntactic_changes: has_changes,
+    };
+}
+
 fn diff_file_content(
     lhs_display_path: &str,
     rhs_display_path: &str,
@@ -300,8 +338,7 @@ fn diff_file_content(
             return DiffResult {
                 lhs_display_path: lhs_display_path.into(),
                 rhs_display_path: rhs_display_path.into(),
-                display_language: None,
-                language_used: None,
+                file_format: FileFormat::Binary,
                 lhs_src: FileContent::Binary,
                 rhs_src: FileContent::Binary,
                 lhs_positions: vec![],
@@ -334,13 +371,17 @@ fn diff_file_content(
     let lang_config = language.map(tsp::from_language);
 
     if lhs_bytes == rhs_bytes {
+        let file_format = match language {
+            Some(language) => FileFormat::SupportedLanguage(language),
+            None => FileFormat::PlainText,
+        };
+
         // If the two files are completely identical, return early
         // rather than doing any more work.
         return DiffResult {
             lhs_display_path: lhs_display_path.into(),
             rhs_display_path: rhs_display_path.into(),
-            display_language: language.map(|l| language_name(l).into()),
-            language_used: language,
+            file_format,
             lhs_src: FileContent::Text("".into()),
             rhs_src: FileContent::Text("".into()),
             lhs_positions: vec![],
@@ -351,127 +392,171 @@ fn diff_file_content(
         };
     }
 
-    let mut language_used = None;
-    let (lang_name, lhs_positions, rhs_positions) = match lang_config {
-        _ if lhs_bytes.len() > diff_options.byte_limit
-            || rhs_bytes.len() > diff_options.byte_limit =>
-        {
+    let (file_format, lhs_positions, rhs_positions) = match lang_config {
+        None => {
+            let file_format = FileFormat::PlainText;
+            if diff_options.check_only {
+                return check_only_text(
+                    &file_format,
+                    lhs_display_path,
+                    rhs_display_path,
+                    &lhs_src,
+                    &rhs_src,
+                );
+            }
+
             let lhs_positions = line_parser::change_positions(&lhs_src, &rhs_src);
             let rhs_positions = line_parser::change_positions(&rhs_src, &lhs_src);
-            (
-                Some("Text (exceeded DFT_BYTE_LIMIT)".into()),
-                lhs_positions,
-                rhs_positions,
-            )
+            (file_format, lhs_positions, rhs_positions)
         }
         Some(ts_lang) => {
-            let lhs_tree = tsp::to_tree(&lhs_src, &ts_lang);
-            let rhs_tree = tsp::to_tree(&rhs_src, &ts_lang);
-
             let arena = Arena::new();
-            let lhs = tsp::to_syntax(
-                &lhs_tree,
-                &lhs_src,
-                &arena,
-                &ts_lang,
-                diff_options.ignore_comments,
-            );
-            let rhs = tsp::to_syntax(
-                &rhs_tree,
-                &rhs_src,
-                &arena,
-                &ts_lang,
-                diff_options.ignore_comments,
-            );
+            match tsp::to_tree_with_limit(diff_options, &ts_lang, &lhs_src, &rhs_src) {
+                Ok((lhs_tree, rhs_tree)) => {
+                    match tsp::to_syntax_with_limit(
+                        &lhs_src,
+                        &rhs_src,
+                        &lhs_tree,
+                        &rhs_tree,
+                        &arena,
+                        &ts_lang,
+                        diff_options,
+                    ) {
+                        Ok((lhs, rhs)) => {
+                            if diff_options.check_only {
+                                let file_format = match language {
+                                    Some(language) => FileFormat::SupportedLanguage(language),
+                                    None => FileFormat::PlainText,
+                                };
 
-            init_all_info(&lhs, &rhs);
+                                let has_syntactic_changes = lhs != rhs;
+                                return DiffResult {
+                                    lhs_display_path: lhs_display_path.into(),
+                                    rhs_display_path: rhs_display_path.into(),
+                                    file_format,
+                                    lhs_src: FileContent::Text(lhs_src),
+                                    rhs_src: FileContent::Text(rhs_src),
+                                    lhs_positions: vec![],
+                                    rhs_positions: vec![],
+                                    hunks: vec![],
+                                    has_byte_changes: true,
+                                    has_syntactic_changes,
+                                };
+                            }
 
-            if diff_options.check_only {
-                let lang_name = language.map(|l| language_name(l).into());
-                let has_syntactic_changes = lhs != rhs;
+                            let mut change_map = ChangeMap::default();
+                            let possibly_changed = if env::var("DFT_DBG_KEEP_UNCHANGED").is_ok() {
+                                vec![(lhs.clone(), rhs.clone())]
+                            } else {
+                                unchanged::mark_unchanged(&lhs, &rhs, &mut change_map)
+                            };
 
-                language_used = language;
-                return DiffResult {
-                    lhs_display_path: lhs_display_path.into(),
-                    rhs_display_path: rhs_display_path.into(),
-                    display_language: lang_name,
-                    language_used,
-                    lhs_src: FileContent::Text(lhs_src),
-                    rhs_src: FileContent::Text(rhs_src),
-                    lhs_positions: vec![],
-                    rhs_positions: vec![],
-                    hunks: vec![],
-                    has_byte_changes: true,
-                    has_syntactic_changes,
-                };
-            }
+                            let mut exceeded_graph_limit = false;
 
-            let mut change_map = ChangeMap::default();
-            let possibly_changed = if env::var("DFT_DBG_KEEP_UNCHANGED").is_ok() {
-                vec![(lhs.clone(), rhs.clone())]
-            } else {
-                unchanged::mark_unchanged(&lhs, &rhs, &mut change_map)
-            };
+                            for (lhs_section_nodes, rhs_section_nodes) in possibly_changed {
+                                init_next_prev(&lhs_section_nodes);
+                                init_next_prev(&rhs_section_nodes);
 
-            let mut exceeded_graph_limit = false;
+                                match mark_syntax(
+                                    lhs_section_nodes.get(0).copied(),
+                                    rhs_section_nodes.get(0).copied(),
+                                    &mut change_map,
+                                    diff_options.graph_limit,
+                                ) {
+                                    Ok(()) => {}
+                                    Err(ExceededGraphLimit {}) => {
+                                        exceeded_graph_limit = true;
+                                        break;
+                                    }
+                                }
+                            }
 
-            for (lhs_section_nodes, rhs_section_nodes) in possibly_changed {
-                init_next_prev(&lhs_section_nodes);
-                init_next_prev(&rhs_section_nodes);
+                            if exceeded_graph_limit {
+                                let lhs_positions =
+                                    line_parser::change_positions(&lhs_src, &rhs_src);
+                                let rhs_positions =
+                                    line_parser::change_positions(&rhs_src, &lhs_src);
+                                (
+                                    FileFormat::TextFallback {
+                                        reason: "exceeded DFT_GRAPH_LIMIT".into(),
+                                    },
+                                    lhs_positions,
+                                    rhs_positions,
+                                )
+                            } else {
+                                // TODO: Make this .expect() unnecessary.
+                                let language = language.expect(
+                                    "If we had a ts_lang, we must have guessed the language",
+                                );
+                                fix_all_sliders(language, &lhs, &mut change_map);
+                                fix_all_sliders(language, &rhs, &mut change_map);
 
-                match mark_syntax(
-                    lhs_section_nodes.get(0).copied(),
-                    rhs_section_nodes.get(0).copied(),
-                    &mut change_map,
-                    diff_options.graph_limit,
-                ) {
-                    Ok(()) => {}
-                    Err(ExceededGraphLimit {}) => {
-                        exceeded_graph_limit = true;
-                        break;
+                                let mut lhs_positions = syntax::change_positions(&lhs, &change_map);
+                                let mut rhs_positions = syntax::change_positions(&rhs, &change_map);
+
+                                if diff_options.ignore_comments {
+                                    let lhs_comments =
+                                        tsp::comment_positions(&lhs_tree, &lhs_src, &ts_lang);
+                                    lhs_positions.extend(lhs_comments);
+
+                                    let rhs_comments =
+                                        tsp::comment_positions(&rhs_tree, &rhs_src, &ts_lang);
+                                    rhs_positions.extend(rhs_comments);
+                                }
+
+                                (
+                                    FileFormat::SupportedLanguage(language),
+                                    lhs_positions,
+                                    rhs_positions,
+                                )
+                            }
+                        }
+                        Err(tsp::ExceededParseErrorLimit(error_count)) => {
+                            let file_format = FileFormat::TextFallback {
+                                reason: format!(
+                                    "{} error{}, exceeded DFT_PARSE_ERROR_LIMIT",
+                                    error_count,
+                                    if error_count == 1 { "" } else { "s" }
+                                ),
+                            };
+
+                            if diff_options.check_only {
+                                return check_only_text(
+                                    &file_format,
+                                    lhs_display_path,
+                                    rhs_display_path,
+                                    &lhs_src,
+                                    &rhs_src,
+                                );
+                            }
+
+                            let lhs_positions = line_parser::change_positions(&lhs_src, &rhs_src);
+                            let rhs_positions = line_parser::change_positions(&rhs_src, &lhs_src);
+                            (file_format, lhs_positions, rhs_positions)
+                        }
                     }
                 }
-            }
+                Err(tsp::ExceededByteLimit(num_bytes)) => {
+                    let file_format = FileFormat::TextFallback {
+                        reason: format!("{} exceeded DFT_BYTE_LIMIT", &format_num_bytes(num_bytes)),
+                    };
 
-            if exceeded_graph_limit {
-                let lhs_positions = line_parser::change_positions(&lhs_src, &rhs_src);
-                let rhs_positions = line_parser::change_positions(&rhs_src, &lhs_src);
-                (
-                    Some("Text (exceeded DFT_GRAPH_LIMIT)".into()),
-                    lhs_positions,
-                    rhs_positions,
-                )
-            } else {
-                language_used = language;
-                // TODO: Make this .expect() unnecessary.
-                let language =
-                    language.expect("If we had a ts_lang, we must have guessed the language");
-                fix_all_sliders(language, &lhs, &mut change_map);
-                fix_all_sliders(language, &rhs, &mut change_map);
+                    if diff_options.check_only {
+                        return check_only_text(
+                            &file_format,
+                            lhs_display_path,
+                            rhs_display_path,
+                            &lhs_src,
+                            &rhs_src,
+                        );
+                    }
 
-                let mut lhs_positions = syntax::change_positions(&lhs, &change_map);
-                let mut rhs_positions = syntax::change_positions(&rhs, &change_map);
-
-                if diff_options.ignore_comments {
-                    let lhs_comments = tsp::comment_positions(&lhs_tree, &lhs_src, &ts_lang);
-                    lhs_positions.extend(lhs_comments);
-
-                    let rhs_comments = tsp::comment_positions(&rhs_tree, &rhs_src, &ts_lang);
-                    rhs_positions.extend(rhs_comments);
+                    let lhs_positions = line_parser::change_positions(&lhs_src, &rhs_src);
+                    let rhs_positions = line_parser::change_positions(&rhs_src, &lhs_src);
+                    (file_format, lhs_positions, rhs_positions)
                 }
-
-                (
-                    Some(language_name(language).into()),
-                    lhs_positions,
-                    rhs_positions,
-                )
             }
         }
-        None => {
-            let lhs_positions = line_parser::change_positions(&lhs_src, &rhs_src);
-            let rhs_positions = line_parser::change_positions(&rhs_src, &lhs_src);
-            (None, lhs_positions, rhs_positions)
-        }
     };
 
     let opposite_to_lhs = opposite_positions(&lhs_positions);
@@ -491,8 +576,7 @@ fn diff_file_content(
     DiffResult {
         lhs_display_path: lhs_display_path.into(),
         rhs_display_path: rhs_display_path.into(),
-        display_language: lang_name,
-        language_used,
+        file_format,
         lhs_src: FileContent::Text(lhs_src),
         rhs_src: FileContent::Text(rhs_src),
         lhs_positions,
@@ -548,10 +632,6 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
         (FileContent::Text(lhs_src), FileContent::Text(rhs_src)) => {
             let hunks = &summary.hunks;
 
-            let display_language = summary
-                .display_language
-                .clone()
-                .unwrap_or_else(|| "Text".into());
             if !summary.has_syntactic_changes {
                 if display_options.print_unchanged {
                     println!(
@@ -561,16 +641,20 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                             &summary.rhs_display_path,
                             1,
                             1,
-                            &display_language,
+                            &summary.file_format,
                             display_options
                         )
                     );
-                    if display_language == "Text" || summary.lhs_src == summary.rhs_src {
-                        // TODO: there are other strings used for text now, so
-                        // they will hit the second case incorrectly.
-                        println!("No changes.\n");
-                    } else {
-                        println!("No syntactic changes.\n");
+                    match summary.file_format {
+                        _ if summary.lhs_src == summary.rhs_src => {
+                            println!("No changes.\n");
+                        }
+                        FileFormat::SupportedLanguage(_) => {
+                            println!("No syntactic changes.\n");
+                        }
+                        _ => {
+                            println!("No changes.\n");
+                        }
                     }
                 }
                 return;
@@ -584,16 +668,17 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         &summary.rhs_display_path,
                         1,
                         1,
-                        &display_language,
+                        &summary.file_format,
                         display_options
                     )
                 );
-                if display_language == "Text" {
-                    // TODO: there are other Text names now, so
-                    // they will hit the second case incorrectly.
-                    println!("Has changes.\n");
-                } else {
-                    println!("Has syntactic changes.\n");
+                match summary.file_format {
+                    FileFormat::SupportedLanguage(_) => {
+                        println!("Has syntactic changes.\n");
+                    }
+                    _ => {
+                        println!("Has changes.\n");
+                    }
                 }
 
                 return;
@@ -610,8 +695,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         hunks,
                         &summary.lhs_display_path,
                         &summary.rhs_display_path,
-                        &display_language,
-                        summary.language_used,
+                        &summary.file_format,
                     );
                 }
                 DisplayMode::SideBySide | DisplayMode::SideBySideShowBoth => {
@@ -620,8 +704,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         display_options,
                         &summary.lhs_display_path,
                         &summary.rhs_display_path,
-                        &display_language,
-                        summary.language_used,
+                        &summary.file_format,
                         lhs_src,
                         rhs_src,
                         &summary.lhs_positions,
@@ -639,7 +722,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         &summary.rhs_display_path,
                         1,
                         1,
-                        "binary",
+                        &FileFormat::Binary,
                         display_options
                     )
                 );
@@ -660,7 +743,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                     &summary.rhs_display_path,
                     1,
                     1,
-                    "binary",
+                    &FileFormat::Binary,
                     display_options
                 )
             );
@@ -693,4 +776,19 @@ mod tests {
         assert_eq!(res.lhs_positions, vec![]);
         assert_eq!(res.rhs_positions, vec![]);
     }
+
+    #[test]
+    fn test_num_bytes_small() {
+        assert_eq!(&format_num_bytes(200), "200B");
+    }
+
+    #[test]
+    fn test_num_bytes_kb() {
+        assert_eq!(&format_num_bytes(10_000), "10KiB");
+    }
+
+    #[test]
+    fn test_num_bytes_mb() {
+        assert_eq!(&format_num_bytes(3_000_000), "3MiB");
+    }
 }
